@@ -27,7 +27,10 @@ use std::time::SystemTime;
 use tokio::sync::mpsc;
 use tokio::sync::oneshot;
 
-type Request = (Vec<ModuleSpecifier>, oneshot::Sender<Result<(), AnyError>>);
+type Request = (
+  Vec<(ModuleSpecifier, deno_graph::ModuleKind)>,
+  oneshot::Sender<Result<(), AnyError>>,
+);
 
 /// A "server" that handles requests from the language server to cache modules
 /// in its own thread.
@@ -47,13 +50,13 @@ impl CacheServer {
     let _join_handle = thread::spawn(move || {
       let runtime = create_basic_runtime();
       runtime.block_on(async {
-        let ps = ProcState::build(Flags {
+        let ps = ProcState::build(Arc::new(Flags {
           cache_path: maybe_cache_path,
           ca_stores: maybe_ca_stores,
           ca_file: maybe_ca_file,
           unsafely_ignore_certificate_errors,
           ..Default::default()
-        })
+        }))
         .await
         .unwrap();
         let maybe_import_map_resolver =
@@ -111,7 +114,7 @@ impl CacheServer {
   /// client.
   pub async fn cache(
     &self,
-    roots: Vec<ModuleSpecifier>,
+    roots: Vec<(ModuleSpecifier, deno_graph::ModuleKind)>,
   ) -> Result<(), AnyError> {
     let (tx, rx) = oneshot::channel::<Result<(), AnyError>>();
     if self.0.send((roots, tx)).is_err() {
