@@ -1,4 +1,4 @@
-// Copyright 2018-2021 the Deno authors. All rights reserved. MIT license.
+// Copyright 2018-2022 the Deno authors. All rights reserved. MIT license.
 
 use std::fs;
 use tempfile::TempDir;
@@ -6,15 +6,31 @@ use test_util as util;
 
 #[test]
 fn branch() {
+  run_coverage_text("branch", "ts");
+}
+
+#[test]
+fn complex() {
+  run_coverage_text("complex", "ts");
+}
+
+#[test]
+fn final_blankline() {
+  run_coverage_text("final_blankline", "js");
+}
+
+fn run_coverage_text(test_name: &str, extension: &str) {
+  let deno_dir = TempDir::new().expect("tempdir fail");
   let tempdir = TempDir::new().expect("tempdir fail");
   let tempdir = tempdir.path().join("cov");
-  let status = util::deno_cmd()
+
+  let status = util::deno_cmd_with_deno_dir(deno_dir.path())
     .current_dir(util::testdata_path())
     .arg("test")
     .arg("--quiet")
     .arg("--unstable")
     .arg(format!("--coverage={}", tempdir.to_str().unwrap()))
-    .arg("coverage/branch_test.ts")
+    .arg(format!("coverage/{}_test.{}", test_name, extension))
     .stdout(std::process::Stdio::piped())
     .stderr(std::process::Stdio::inherit())
     .status()
@@ -22,23 +38,25 @@ fn branch() {
 
   assert!(status.success());
 
-  let output = util::deno_cmd()
+  let output = util::deno_cmd_with_deno_dir(deno_dir.path())
     .current_dir(util::testdata_path())
     .arg("coverage")
-    .arg("--quiet")
     .arg("--unstable")
     .arg(format!("{}/", tempdir.to_str().unwrap()))
     .stdout(std::process::Stdio::piped())
-    .stderr(std::process::Stdio::inherit())
+    .stderr(std::process::Stdio::piped())
     .output()
     .expect("failed to spawn coverage reporter");
 
+  // Verify there's no "Check" being printed
+  assert!(output.stderr.is_empty());
+
   let actual =
     util::strip_ansi_codes(std::str::from_utf8(&output.stdout).unwrap())
       .to_string();
 
   let expected = fs::read_to_string(
-    util::testdata_path().join("coverage/expected_branch.out"),
+    util::testdata_path().join(format!("coverage/{}_expected.out", test_name)),
   )
   .unwrap();
 
@@ -50,7 +68,7 @@ fn branch() {
 
   assert!(output.status.success());
 
-  let output = util::deno_cmd()
+  let output = util::deno_cmd_with_deno_dir(deno_dir.path())
     .current_dir(util::testdata_path())
     .arg("coverage")
     .arg("--quiet")
@@ -67,7 +85,7 @@ fn branch() {
       .to_string();
 
   let expected = fs::read_to_string(
-    util::testdata_path().join("coverage/expected_branch.lcov"),
+    util::testdata_path().join(format!("coverage/{}_expected.lcov", test_name)),
   )
   .unwrap();
 
@@ -81,15 +99,18 @@ fn branch() {
 }
 
 #[test]
-fn complex() {
+fn multifile_coverage() {
+  let deno_dir = TempDir::new().expect("tempdir fail");
   let tempdir = TempDir::new().expect("tempdir fail");
-  let status = util::deno_cmd()
+  let tempdir = tempdir.path().join("cov");
+
+  let status = util::deno_cmd_with_deno_dir(deno_dir.path())
     .current_dir(util::testdata_path())
     .arg("test")
     .arg("--quiet")
     .arg("--unstable")
-    .arg(format!("--coverage={}", tempdir.path().to_str().unwrap()))
-    .arg("coverage/complex_test.ts")
+    .arg(format!("--coverage={}", tempdir.to_str().unwrap()))
+    .arg("coverage/multifile/")
     .stdout(std::process::Stdio::piped())
     .stderr(std::process::Stdio::inherit())
     .status()
@@ -97,23 +118,25 @@ fn complex() {
 
   assert!(status.success());
 
-  let output = util::deno_cmd()
+  let output = util::deno_cmd_with_deno_dir(deno_dir.path())
     .current_dir(util::testdata_path())
     .arg("coverage")
-    .arg("--quiet")
     .arg("--unstable")
-    .arg(format!("{}/", tempdir.path().to_str().unwrap()))
+    .arg(format!("{}/", tempdir.to_str().unwrap()))
     .stdout(std::process::Stdio::piped())
-    .stderr(std::process::Stdio::inherit())
+    .stderr(std::process::Stdio::piped())
     .output()
     .expect("failed to spawn coverage reporter");
 
+  // Verify there's no "Check" being printed
+  assert!(output.stderr.is_empty());
+
   let actual =
     util::strip_ansi_codes(std::str::from_utf8(&output.stdout).unwrap())
       .to_string();
 
   let expected = fs::read_to_string(
-    util::testdata_path().join("coverage/expected_complex.out"),
+    util::testdata_path().join("coverage/multifile/expected.out"),
   )
   .unwrap();
 
@@ -125,13 +148,13 @@ fn complex() {
 
   assert!(output.status.success());
 
-  let output = util::deno_cmd()
+  let output = util::deno_cmd_with_deno_dir(deno_dir.path())
     .current_dir(util::testdata_path())
     .arg("coverage")
     .arg("--quiet")
     .arg("--unstable")
     .arg("--lcov")
-    .arg(format!("{}/", tempdir.path().to_str().unwrap()))
+    .arg(format!("{}/", tempdir.to_str().unwrap()))
     .stdout(std::process::Stdio::piped())
     .stderr(std::process::Stdio::inherit())
     .output()
@@ -142,7 +165,7 @@ fn complex() {
       .to_string();
 
   let expected = fs::read_to_string(
-    util::testdata_path().join("coverage/expected_complex.lcov"),
+    util::testdata_path().join("coverage/multifile/expected.lcov"),
   )
   .unwrap();
 
